"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpService = void 0;
const core_1 = require("@message-queue-toolkit/core");
class AbstractAmqpService extends core_1.AbstractQueueService {
    connection;
    // @ts-ignore
    channel;
    isShuttingDown;
    constructor(dependencies, options) {
        super(dependencies, options);
        this.connection = dependencies.amqpConnection;
        this.isShuttingDown = false;
    }
    async destroyConnection() {
        if (this.channel) {
            try {
                await this.channel.close();
            }
            finally {
                // @ts-ignore
                this.channel = undefined;
            }
        }
    }
    async init() {
        this.isShuttingDown = false;
        // If channel exists, recreate it
        if (this.channel) {
            this.isShuttingDown = true;
            await this.destroyConnection();
            this.isShuttingDown = false;
        }
        this.channel = await this.connection.createChannel();
        this.channel.on('close', () => {
            if (!this.isShuttingDown) {
                this.logger.error(`AMQP connection lost!`);
                this.init().catch((err) => {
                    this.handleError(err);
                    throw err;
                });
            }
        });
        this.channel.on('error', (err) => {
            this.handleError(err);
        });
        if (!this.queueLocator) {
            await this.channel.assertQueue(this.queueName, this.queueConfiguration);
        }
        else {
            // queue check breaks channel if not successful
            const checkChannel = await this.connection.createChannel();
            checkChannel.on('error', () => {
                // it's OK
            });
            try {
                await checkChannel.checkQueue(this.queueLocator.queueName);
                await checkChannel.close();
            }
            catch (err) {
                throw new Error(`Queue with queueName ${this.queueLocator.queueName} does not exist.`);
            }
        }
    }
    async close() {
        this.isShuttingDown = true;
        await this.destroyConnection();
    }
}
exports.AbstractAmqpService = AbstractAmqpService;
//# sourceMappingURL=AbstractAmqpService.js.map